const { parse } = require("svg-parser");

class EdupageParser {
  constructor(svg) {
    this.parsedSVG = parse(svg);
    this.svgData = this.parsedSVG.children[0].children[1].children;

    this.daysY = {
      mondayY: 420,
      tuesdayY: 726,
      wednesdayY: 1032,
      thursdayY: 1338,
      fridayY: 1644
    };

    this.timesX = this.svgData
      .filter(child => child.tagName === "text")
      .filter(text => text.properties.y === 335)
      .map(time => ({ time: time.children[0].value, x: time.properties.x }));
  }

  getDay(y) {
    switch (y) {
      case this.daysY.mondayY:
        return "Monday";
      case this.daysY.tuesdayY:
        return "Tuesday";
      case this.daysY.wednesdayY:
        return "Wednesday";
      case this.daysY.thursdayY:
        return "Thursday";
      case this.daysY.fridayY:
        return "Friday";

      default:
        return "";
    }
  }

  getTime(xStart, xEnd) {
    const times = [];

    for (const time of this.timesX) {
      if (xStart <= time.x && xEnd >= time.x) {
        times.push(time.time);
      }
    }

    const startTime = times[0].split("-")[0];
    const endTime = times[times.length - 1].split("-")[1];

    return { startTime, endTime };
  }

  isTeacherTimetable() {
    const title = this.svgData[0].children[0].value;

    return title.startsWith("Teacher ");
  }

  parseTimetable() {
    const groupName = this.svgData[0].children[0].value;
    const timeTable = {};

    const subjects = this.svgData
      .filter(child => child.tagName === "rect")
      .map(subject => ({
        xStart: subject.properties.x,
        xEnd: subject.properties.x + subject.properties.width,

        y: subject.properties.y,
        subjectInfo: subject.children[0].children[0].value
      }));

    for (const subject of subjects) {
      const [subjectName, teacherName, room] = subject.subjectInfo.split("\n");
      const { startTime, endTime } = this.getTime(subject.xStart, subject.xEnd);
      const [sectionNumber] = subjectName.match(/\(\d+?\)/);
      const day = this.getDay(subject.y);

      const oneTimeTable = {
        subjectName: subjectName.split(" (")[0].replace("(", " ("),
        subjectNameShort: subjectName
          .split("(")[0]
          .split(" ")
          .map(subject => subject.charAt(0))
          .join(""),
        teacherName,
        teacherNameShort: teacherName
          .split(" ")
          .map(subject => subject.charAt(0))
          .join(""),
        sectionNumber: Number(sectionNumber.match(/\d/)[0]),
        startTime,
        endTime,
        room,
        day
      };

      if (timeTable[day]) {
        timeTable[day].push(oneTimeTable);
      } else {
        timeTable[day] = [oneTimeTable];
      }
    }

    return {
      groupName,
      timeTable
    };
  }

  parseTeacher() {
    let teacherSubjectName;

    const timeTable = {};
    const teacherName = this.svgData[0].children[0].value.replace("Teacher ", "");
    const teacherNameShort = teacherName
      .split(" ")
      .map(el => el.charAt(0))
      .join("");

    const subjects = this.svgData
      .filter(child => child.tagName === "rect")
      .map(subject => ({
        xStart: subject.properties.x,
        xEnd: subject.properties.x + subject.properties.width,

        y: subject.properties.y,
        subjectInfo: subject.children[0].children[0].value
      }));

    subjects.forEach((el, index) => {
      const [subjectName, groups, room] = el.subjectInfo.split("\n");
      const { startTime, endTime } = this.getTime(el.xStart, el.xEnd);
      const [sectionNumber] = subjectName.match(/(\d)/);
      const day = this.getDay(el.y);

      const oneTimeTable = {
        subjectName: subjectName.split(" (")[0],
        subjectNameShort: subjectName
          .split(" (")[0]
          .split(" ")
          .map(el => el.charAt(0))
          .join(""),
        groups: groups.split("/"),
        sectionNumber: Number(sectionNumber),
        startTime,
        endTime,
        room,
        day
      };

      if (timeTable[day]) {
        timeTable[day].push(oneTimeTable);
      } else {
        timeTable[day] = [oneTimeTable];
      }

      if (index === subjects.length - 1) {
        teacherSubjectName = oneTimeTable.subjectName;
      }
    });

    return {
      teacherName,
      teacherNameShort,
      subjectName: teacherSubjectName,
      subjectNameShort: teacherSubjectName
        .split(" ")
        .map(el => el.charAt(0))
        .join(""),
      timeTable
    };
  }
}

module.exports = EdupageParser;
