const fs = require("fs");

const EdupageParser = require("../");

describe("Parse timetable-example-1.svg", () => {
  const svg = fs.readFileSync("test/timetable-example-1.svg", "utf8");

  const edupageParser = new EdupageParser(svg);
  const parsedTimetable = edupageParser.parseTimetable();

  const groupName = "CIE-19-19";
  const expectedDays = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday"];
  const numberOfSubjectsInEachDay = {
    Monday: 2,
    Tuesday: 3,
    Wednesday: 4,
    Thursday: 1,
    Friday: 3
  };

  test("Should not be teacher timetable", () => {
    expect(edupageParser.isTeacherTimetable()).toBe(false);
  });

  test("Should return right groupname", () => {
    expect(parsedTimetable.groupName).toMatch(groupName);
  });

  test(`Should contain right ${expectedDays.length} days`, () => {
    expect(Object.keys(parsedTimetable.timeTable)).toEqual(expect.arrayContaining(expectedDays));
  });

  for (const day of expectedDays) {
    test(`In ${day} should be ${numberOfSubjectsInEachDay[day]} subjects`, () => {
      expect(parsedTimetable.timeTable[day].length).toBe(numberOfSubjectsInEachDay[day]);
    });

    for (const subject of parsedTimetable.timeTable[day]) {
      const shortenedSubjectName = subject.subjectName
        .split(" ")
        .map(el => el.charAt(0))
        .join("");

      test(`Subject name "${subject.subjectName}" should be "${shortenedSubjectName}"`, () => {
        expect(subject.subjectNameShort).toMatch(shortenedSubjectName);
      });

      const shortenedTeacherName = subject.teacherName
        .split(" ")
        .map(el => el.charAt(0))
        .join("");

      test(`Subject name "${subject.teacherName}" should be "${shortenedTeacherName}"`, () => {
        expect(subject.teacherNameShort).toMatch(shortenedTeacherName);
      });
    }
  }
});

describe("Parse timetable-example-2.svg", () => {
  const svg = fs.readFileSync("test/timetable-example-2.svg", "utf8");

  const edupageParser = new EdupageParser(svg);
  const parsedTimetable = edupageParser.parseTimetable();

  const basicKoreanSection = 9;
  const groupName = "CIE-18-14";
  const academicEnglishSection = 9;
  const expectedDays = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday"];
  const numberOfSubjectsInEachDay = {
    Monday: 3,
    Tuesday: 1,
    Wednesday: 3,
    Thursday: 4,
    Friday: 2
  };

  test("Should not be teacher timetable", () => {
    expect(edupageParser.isTeacherTimetable()).toBe(false);
  });

  test("Should return right groupname", () => {
    expect(parsedTimetable.groupName).toMatch(groupName);
  });

  test(`Should contain right ${expectedDays.length} days`, () => {
    expect(Object.keys(parsedTimetable.timeTable)).toEqual(expect.arrayContaining(expectedDays));
  });

  for (const day of expectedDays) {
    test(`In ${day} should be ${numberOfSubjectsInEachDay[day]} subjects`, () => {
      expect(parsedTimetable.timeTable[day].length).toBe(numberOfSubjectsInEachDay[day]);
    });

    for (const subject of parsedTimetable.timeTable[day]) {
      const shortenedSubjectName = subject.subjectName
        .split(" ")
        .map(el => el.charAt(0))
        .join("");

      test(`Subject name "${subject.subjectName}" should be "${shortenedSubjectName}"`, () => {
        expect(subject.subjectNameShort).toMatch(shortenedSubjectName);
      });

      if (shortenedSubjectName === "AE3") {
        test(`Section for ${subject.subjectName} should be ${academicEnglishSection}`, () => {
          expect(subject.sectionNumber).toBe(academicEnglishSection);
        });
      }

      if (shortenedSubjectName === "BK1") {
        test(`Section for ${subject.subjectName} should be ${basicKoreanSection}`, () => {
          expect(subject.sectionNumber).toBe(basicKoreanSection);
        });
      }

      const shortenedTeacherName = subject.teacherName
        .split(" ")
        .map(el => el.charAt(0))
        .join("");

      test(`Subject name "${subject.teacherName}" should be "${shortenedTeacherName}"`, () => {
        expect(subject.teacherNameShort).toMatch(shortenedTeacherName);
      });
    }
  }
});

describe("Parse timetable-example-3.svg", () => {
  const svg = fs.readFileSync("test/timetable-example-3.svg", "utf8");

  const edupageParser = new EdupageParser(svg);
  const parsedTimetable = edupageParser.parseTimetable();

  const groupName = "CSE-17-01";
  const expectedDays = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday"];
  const numberOfSubjectsInEachDay = {
    Monday: 3,
    Tuesday: 3,
    Wednesday: 3,
    Thursday: 2,
    Friday: 2
  };

  test("Should not be teacher timetable", () => {
    expect(edupageParser.isTeacherTimetable()).toBe(false);
  });

  test("Should return right groupname", () => {
    expect(parsedTimetable.groupName).toMatch(groupName);
  });

  test(`Should contain right ${expectedDays.length} days`, () => {
    expect(Object.keys(parsedTimetable.timeTable)).toEqual(expect.arrayContaining(expectedDays));
  });

  for (const day of expectedDays) {
    test(`In ${day} should be ${numberOfSubjectsInEachDay[day]} subjects`, () => {
      expect(parsedTimetable.timeTable[day].length).toBe(numberOfSubjectsInEachDay[day]);
    });

    for (const subject of parsedTimetable.timeTable[day]) {
      const shortenedSubjectName = subject.subjectName
        .split("(")[0]
        .split(" ")
        .map(el => el.charAt(0))
        .join("");

      test(`Subject name "${subject.subjectName}" should be "${shortenedSubjectName}"`, () => {
        expect(subject.subjectNameShort).toMatch(shortenedSubjectName);
      });

      const shortenedTeacherName = subject.teacherName
        .split(" ")
        .map(el => el.charAt(0))
        .join("");

      test(`Subject name "${subject.teacherName}" should be "${shortenedTeacherName}"`, () => {
        expect(subject.teacherNameShort).toMatch(shortenedTeacherName);
      });
    }
  }
});

describe("Parse teacher-example.svg", () => {
  const svg = fs.readFileSync("test/teacher-example.svg", "utf8");

  const edupageParser = new EdupageParser(svg);
  const parsedTeacher = edupageParser.parseTeacher();

  const teacherName = "Khalilova Alie";
  const teacherNameShort = "KA";
  const subjectName = "Academic English 2";
  const subjectNameShort = "AE2";
  const expectedDays = ["Tuesday", "Wednesday", "Thursday", "Friday"];
  const numberOfSubjectsInEachDay = {
    Tuesday: 3,
    Wednesday: 1,
    Thursday: 4,
    Friday: 4
  };

  test("Should not be teacher timetable", () => {
    expect(edupageParser.isTeacherTimetable()).toBe(true);
  });

  test("Should return right teacher name", () => {
    expect(parsedTeacher.teacherName).toMatch(teacherName);
  });

  test("Should return right shortened teacher name", () => {
    expect(parsedTeacher.teacherNameShort).toMatch(teacherNameShort);
  });

  test("Should return right subject name", () => {
    expect(parsedTeacher.subjectName).toMatch(subjectName);
  });

  test("Should return right shortened subject name", () => {
    expect(parsedTeacher.subjectNameShort).toMatch(subjectNameShort);
  });

  test(`Should contain right ${expectedDays.length} days`, () => {
    expect(Object.keys(parsedTeacher.timeTable)).toEqual(expect.arrayContaining(expectedDays));
  });

  for (const day of expectedDays) {
    test(`In ${day} should be ${numberOfSubjectsInEachDay[day]} subjects`, () => {
      expect(parsedTeacher.timeTable[day].length).toBe(numberOfSubjectsInEachDay[day]);
    });
  }
});
